@@ -1,17 +1,18 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the demonstration applications of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -21,8 +22,8 @@
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
@@ -33,8 +34,7 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
+**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -152,6 +152,9 @@ void MenuManager::itemSelected(int userCode, const QString &menuName)
     case LAUNCH:
         this->launchExample(this->currentInfo);
         break;
+    case LAUNCH_QML:
+        this->launchQmlExample(this->currentInfo);
+        break;
     case DOCUMENTATION:
         this->showDocInAssistant(this->currentInfo);
         break;
@@ -169,6 +172,8 @@ void MenuManager::itemSelected(int userCode, const QString &menuName)
         this->score->queueMovie(this->currentInfo + " -out");
         this->score->queueMovie(this->currentInfo + " -buttons -out", Score::NEW_ANIMATION_ONLY);
         this->score->queueMovie("back -out", Score::ONLY_IF_VISIBLE);
+        if(qmlRoot)
+            qmlRoot->setProperty("show", QVariant(false));
         // book-keeping:
         this->currentMenuCode = ROOT;
         this->currentMenu = menuName + " -menu1";
@@ -191,6 +196,8 @@ void MenuManager::itemSelected(int userCode, const QString &menuName)
         this->score->queueMovie(this->currentMenu + " -out", Score::FROM_START, Score::LOCK_ITEMS);
         this->score->queueMovie(this->currentMenuButtons + " -out", Score::FROM_START, Score::LOCK_ITEMS);
         this->score->queueMovie(this->currentInfo + " -out");
+        if(qmlRoot)
+            qmlRoot->setProperty("show", QVariant(false));
         // book-keeping:
         this->currentMenuCode = MENU1;
         this->currentCategory = menuName;
@@ -208,6 +215,8 @@ void MenuManager::itemSelected(int userCode, const QString &menuName)
         // out:
         this->score->queueMovie(this->currentInfo + " -out", Score::NEW_ANIMATION_ONLY);
         this->score->queueMovie(this->currentInfo + " -buttons -out", Score::NEW_ANIMATION_ONLY);
+        if(qmlRoot)
+            qmlRoot->setProperty("show", QVariant(false));
         // book-keeping:
         this->currentMenuCode = MENU2;
         this->currentInfo = menuName;
@@ -242,6 +251,8 @@ void MenuManager::itemSelected(int userCode, const QString &menuName)
             // out:
             this->score->queueMovie(this->currentInfo + " -out", Score::NEW_ANIMATION_ONLY);
             this->score->queueMovie(this->currentInfo + " -buttons -out", Score::NEW_ANIMATION_ONLY);
+            if(qmlRoot)
+                qmlRoot->setProperty("show", QVariant(false));
             // book-keeping:
             this->currentMenuCode = MENU1;
             this->currentMenuButtons = this->currentCategory + " -buttons";
@@ -302,7 +313,7 @@ void MenuManager::showDocInAssistant(const QString &name)
     // Send command through remote control even if the process
     // was started to activate assistant and bring it to front:
     QTextStream str(&this->assistantProcess);
-    str << "SetSource " << url << QLatin1Char('\0') << endl;
+    str << "SetSource " << url << QLatin1Char('\n') << endl;
 }
 
 void MenuManager::launchExample(const QString &name)
@@ -343,6 +354,46 @@ void MenuManager::launchExample(const QString &name)
 #endif
 }
 
+void MenuManager::launchQmlExample(const QString &name)
+{
+#ifndef QT_NO_DECLARATIVE
+    if(!qmlRoot){
+        exampleError(QProcess::UnknownError);
+        return;
+    }
+    //resolveQmlFilename - refactor to separate fn?
+    QString dirName = this->info[name]["dirname"];
+    QString category = this->info[name]["category"];
+    QString fileName = this->info[name]["filename"];
+    QDir dir;
+    if (category == "demos")
+        dir = QDir(QLibraryInfo::location(QLibraryInfo::DemosPath));
+    else
+        dir = QDir(QLibraryInfo::location(QLibraryInfo::ExamplesPath));
+    QFile file(dir.path() + "/" + dirName + "/" + fileName + "/" + "main.qml");
+    if(!file.exists()){
+        //try dirname.qml as well
+        file.setFileName(dir.path() + "/" + dirName + "/" + fileName + "/" + fileName.split('/').last() + ".qml");
+        if(!file.exists()){
+            exampleError(QProcess::UnknownError);
+            return;
+        }
+    }
+
+    qmlRoot->setProperty("qmlFile", QVariant(""));//unload component
+    qmlRoot->setProperty("show", QVariant(true));
+    qmlRoot->setProperty("qmlFile", QUrl::fromLocalFile(file.fileName()));
+#else
+    exampleError(QProcess::UnknownError);
+#endif
+}
+
+void MenuManager::quitQML()
+{
+    if(qmlRoot)
+        qmlRoot->setProperty("show", QVariant(false));
+}
+
 void MenuManager::exampleFinished()
 {
 }
@@ -385,6 +436,39 @@ void MenuManager::init(MainWindow *window)
 
         level2MenuNode = level2MenuNode.nextSibling();
     }
+
+    qmlRoot = 0;
+#ifndef QT_NO_DECLARATIVE
+    // Create QML Loader
+    declarativeEngine = new QDeclarativeEngine(this);
+    connect(declarativeEngine, SIGNAL(quit()),
+            this, SLOT(quitQML()));
+
+    QDeclarativeComponent component(declarativeEngine, QUrl("qrc:qml/qmlShell.qml"), this);
+    QDeclarativeItem* qmlRootItem = 0;
+    if(component.isReady()){
+        qmlRoot = component.create();
+        qmlRootItem = qobject_cast<QDeclarativeItem*>(qmlRoot);
+    }else{
+        qDebug() << component.status() << component.errorString();
+    }
+
+    if(qmlRootItem){
+        qmlRootItem->setHeight(this->window->scene->sceneRect().height());
+        qmlRootItem->setWidth(this->window->scene->sceneRect().width());
+        qmlRootItem->setZValue(101);//Above other items
+        qmlRootItem->setCursor(Qt::ArrowCursor);
+        window->scene->addItem(qmlRootItem);
+
+        //Note that QML adds key handling to the app.
+        window->viewport()->setFocusPolicy(Qt::NoFocus);//Correct keyboard focus handling
+        window->setFocusPolicy(Qt::StrongFocus);
+        window->scene->setStickyFocus(true);
+        window->setFocus();
+    }else{
+        qDebug() << "Error initializing QML subsystem, Declarative examples will not work";
+    }
+#endif
 }
 
 void MenuManager::readInfoAboutExample(const QDomElement &example)
@@ -392,13 +476,14 @@ void MenuManager::readInfoAboutExample(const QDomElement &example)
     QString name = example.attribute("name");
     if (this->info.contains(name))
         qWarning() << "__WARNING: MenuManager::readInfoAboutExample: Demo/example with name"
-                    << name << "appears twize in the xml-file!__";
+                    << name << "appears twice in the xml-file!__";
 
     this->info[name]["filename"] = example.attribute("filename");
     this->info[name]["category"] = example.parentNode().toElement().tagName();
     this->info[name]["dirname"] = example.parentNode().toElement().attribute("dirname");
     this->info[name]["changedirectory"] = example.attribute("changedirectory");
     this->info[name]["image"] = example.attribute("image");
+    this->info[name]["qml"] = example.attribute("qml");
 }
 
 QString MenuManager::resolveDataDir(const QString &name)
@@ -433,15 +518,28 @@ QString MenuManager::resolveExeFile(const QString &name)
     dir.cd(dirName);
     dir.cd(fileName);
 
-    QFile unixFile(dir.path() + "/" + fileName);
-    if (unixFile.exists()) return unixFile.fileName();
-    QFile winR(dir.path() + "\\release\\" + fileName + ".exe");
-    if (winR.exists()) return winR.fileName();
-    QFile winD(dir.path() + "\\debug\\" + fileName + ".exe");
-    if (winD.exists()) return winD.fileName();
-    QFile mac(dir.path() + "/" + fileName + ".app");
-    if (mac.exists()) return mac.fileName();
-
+    fileName = fileName.split(QLatin1Char('/')).last();
+#ifdef Q_OS_WIN
+    fileName += QLatin1String(".exe");
+#endif
+    // UNIX, Mac non-framework and Windows installed builds.
+    const QFile installedFile(dir.path() + QLatin1Char('/') + fileName);
+    if (installedFile.exists())
+        return installedFile.fileName();
+    // Windows in-source builds
+#if defined(Q_OS_WIN)
+    const QFile winR(dir.path() + QLatin1String("/release/") + fileName);
+    if (winR.exists())
+        return winR.fileName();
+    const QFile winD(dir.path() + QLatin1String("/debug/") + fileName);
+    if (winD.exists())
+        return winD.fileName();
+#elif defined(Q_OS_MAC)
+    // Mac frameworks
+    const QFile mac(dir.path() + QLatin1Char('/') + fileName + QLatin1String(".app"));
+    if (mac.exists())
+        return mac.fileName();
+#endif
     if (Colors::verbose)
         qDebug() << "- WARNING: Could not resolve executable:" << dir.path() << fileName;
     return "__executable not found__";
@@ -454,7 +552,7 @@ QString MenuManager::resolveDocUrl(const QString &name)
     QString fileName = this->info[name]["filename"];
 
     if (category == "demos")
-        return this->helpRootUrl + "demos-" + fileName + ".html";
+        return this->helpRootUrl + "demos-" + fileName.replace("/", "-") + ".html";
     else
         return this->helpRootUrl + dirName.replace("/", "-") + "-" + fileName + ".html";
 }
@@ -474,6 +572,9 @@ QByteArray MenuManager::getImage(const QString &name)
     QString imageName = this->info[name]["image"];
     QString category = this->info[name]["category"];
     QString fileName = this->info[name]["filename"];
+    bool qml = (this->info[name]["qml"] == QLatin1String("true"));
+    if(qml)
+        fileName = QLatin1String("qml-") + fileName.split('/').last();
 
     if (imageName.isEmpty()){
         if (category == "demos")
@@ -493,7 +594,7 @@ void MenuManager::createRootMenu(const QDomElement &el)
 {
     QString name = el.attribute("name");
     createMenu(el, MENU1);
-    createInfo(new MenuContentItem(el, this->window->scene, 0), name + " -info");
+    createInfo(new MenuContentItem(el, this->window->scene, this->window->mainSceneRoot), name + " -info");
 
     Movie *menuButtonsIn = this->score->insertMovie(name + " -buttons");
     Movie *menuButtonsOut = this->score->insertMovie(name + " -buttons -out");
@@ -505,19 +606,21 @@ void MenuManager::createSubMenu(const QDomElement &el)
 {
     QString name = el.attribute("name");
     createMenu(el, MENU2);
-    createInfo(new MenuContentItem(el, this->window->scene, 0), name + " -info");
+    createInfo(new MenuContentItem(el, this->window->scene, this->window->mainSceneRoot), name + " -info");
 }
 
 void MenuManager::createLeafMenu(const QDomElement &el)
 {
     QString name = el.attribute("name");
-    createInfo(new ExampleContent(name, this->window->scene, 0), name);
+    createInfo(new ExampleContent(name, this->window->scene, this->window->mainSceneRoot), name);
 
     Movie *infoButtonsIn = this->score->insertMovie(name + " -buttons");
     Movie *infoButtonsOut = this->score->insertMovie(name + " -buttons -out");
     createLowRightLeafButton("Documentation", 600, DOCUMENTATION, infoButtonsIn, infoButtonsOut, 0);
     if (el.attribute("executable") != "false")
         createLowRightLeafButton("Launch", 405, LAUNCH, infoButtonsIn, infoButtonsOut, 0);
+    else if(el.attribute("qml") == "true")
+        createLowRightLeafButton("Display", 405, LAUNCH_QML, infoButtonsIn, infoButtonsOut, 0);
 }
 
 void MenuManager::createMenu(const QDomElement &category, BUTTON_TYPE type)
@@ -546,7 +649,7 @@ void MenuManager::createMenu(const QDomElement &category, BUTTON_TYPE type)
 
             // create normal menu button
             QString label = currentNode.toElement().attribute("name");
-            item = new TextButton(label, TextButton::LEFT, type, this->window->scene, 0);
+            item = new TextButton(label, TextButton::LEFT, type, this->window->scene, this->window->mainSceneRoot);
             currentNode = currentNode.nextSibling();
 
 #ifndef QT_OPENGL_SUPPORT
@@ -646,7 +749,7 @@ void MenuManager::createMenu(const QDomElement &category, BUTTON_TYPE type)
 void MenuManager::createLowLeftButton(const QString &label, BUTTON_TYPE type,
     Movie *movieIn, Movie *movieOut, Movie *movieShake, const QString &menuString)
 {
-    TextButton *button = new TextButton(label, TextButton::RIGHT, type, this->window->scene, 0, TextButton::PANEL);
+    TextButton *button = new TextButton(label, TextButton::RIGHT, type, this->window->scene, this->window->mainSceneRoot, TextButton::PANEL);
     if (!menuString.isNull())
         button->setMenuString(menuString);
     button->setRecursiveVisible(false);
@@ -688,7 +791,7 @@ void MenuManager::createLowLeftButton(const QString &label, BUTTON_TYPE type,
 
 void MenuManager::createLowRightButton(const QString &label, BUTTON_TYPE type, Movie *movieIn, Movie *movieOut, Movie * /*movieShake*/)
 {
-    TextButton *item = new TextButton(label, TextButton::RIGHT, type, this->window->scene, 0, TextButton::PANEL);
+    TextButton *item = new TextButton(label, TextButton::RIGHT, type, this->window->scene, this->window->mainSceneRoot, TextButton::PANEL);
     item->setRecursiveVisible(false);
     item->setZValue(10);
 
@@ -715,7 +818,7 @@ void MenuManager::createLowRightButton(const QString &label, BUTTON_TYPE type, M
 
 void MenuManager::createLowRightLeafButton(const QString &label, int xOffset, BUTTON_TYPE type, Movie *movieIn, Movie *movieOut, Movie * /*movieShake*/)
 {
-    TextButton *item = new TextButton(label, TextButton::RIGHT, type, this->window->scene, 0, TextButton::PANEL);
+    TextButton *item = new TextButton(label, TextButton::RIGHT, type, this->window->scene, this->window->mainSceneRoot, TextButton::PANEL);
     item->setRecursiveVisible(false);
     item->setZValue(10);
 
@@ -831,12 +934,12 @@ void MenuManager::createUpnDownButtons()
     float xOffset = 15.0f;
     float yOffset = 450.0f;
 
-    this->upButton = new TextButton("", TextButton::LEFT, MenuManager::UP, this->window->scene, 0, TextButton::UP);
+    this->upButton = new TextButton("", TextButton::LEFT, MenuManager::UP, this->window->scene, this->window->mainSceneRoot, TextButton::UP);
     this->upButton->prepare();
     this->upButton->setPos(xOffset, yOffset);
     this->upButton->setState(TextButton::DISABLED);
 
-    this->downButton = new TextButton("", TextButton::LEFT, MenuManager::DOWN, this->window->scene, 0, TextButton::DOWN);
+    this->downButton = new TextButton("", TextButton::LEFT, MenuManager::DOWN, this->window->scene, this->window->mainSceneRoot, TextButton::DOWN);
     this->downButton->prepare();
     this->downButton->setPos(xOffset + 10 + this->downButton->sceneBoundingRect().width(), yOffset);
 
