@@ -1,17 +1,18 @@
 /****************************************************************************
 **
-** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the QtDeclarative module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -21,8 +22,8 @@
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
@@ -33,8 +34,7 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
+**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -48,7 +48,8 @@
 QT_BEGIN_NAMESPACE
 
 QDeclarativeLoaderPrivate::QDeclarativeLoaderPrivate()
-    : item(0), component(0), ownComponent(false)
+    : item(0), component(0), ownComponent(false), updatingSize(false),
+      itemWidthValid(false), itemHeightValid(false)
 {
 }
 
@@ -58,8 +59,13 @@ QDeclarativeLoaderPrivate::~QDeclarativeLoaderPrivate()
 
 void QDeclarativeLoaderPrivate::itemGeometryChanged(QDeclarativeItem *resizeItem, const QRectF &newGeometry, const QRectF &oldGeometry)
 {
-    if (resizeItem == item)
+    if (resizeItem == item) {
+        if (!updatingSize && newGeometry.width() != oldGeometry.width())
+            itemWidthValid = true;
+        if (!updatingSize && newGeometry.height() != oldGeometry.height())
+            itemHeightValid = true;
         _q_updateSize(false);
+    }
     QDeclarativeItemChangeListener::itemGeometryChanged(resizeItem, newGeometry, oldGeometry);
 }
 
@@ -99,6 +105,10 @@ void QDeclarativeLoaderPrivate::initResize()
         QDeclarativeItemPrivate *p =
                 static_cast<QDeclarativeItemPrivate *>(QGraphicsItemPrivate::get(qmlItem));
         p->addItemChangeListener(this, QDeclarativeItemPrivate::Geometry);
+        // We may override the item's size, so we need to remember
+        // whether the item provided its own valid size.
+        itemWidthValid = p->widthValid;
+        itemHeightValid = p->heightValid;
     } else if (item && item->isWidget()) {
         QGraphicsWidget *widget = static_cast<QGraphicsWidget*>(item);
         widget->installEventFilter(q);
@@ -129,15 +139,41 @@ void QDeclarativeLoaderPrivate::initResize()
 
     The loaded item can be accessed using the \l item property.
 
-    Loader is like any other visual item and must be positioned and sized 
-    accordingly to become visible. Once the component is loaded, the Loader 
-    is automatically resized to the size of the component.
-
     If the \l source or \l sourceComponent changes, any previously instantiated
     items are destroyed. Setting \l source to an empty string or setting
     \l sourceComponent to \c undefined destroys the currently loaded item,
     freeing resources and leaving the Loader empty.
 
+    \section2 Loader sizing behavior
+
+    Loader is like any other visual item and must be positioned and sized
+    accordingly to become visible.
+
+    \list
+    \o If an explicit size is not specified for the Loader, the Loader
+    is automatically resized to the size of the loaded item once the
+    component is loaded.
+    \o If the size of the Loader is specified explicitly by setting
+    the width, height or by anchoring, the loaded item will be resized
+    to the size of the Loader.
+    \endlist
+
+    In both scenarios the size of the item and the Loader are identical.
+    This ensures that anchoring to the Loader is equivalent to anchoring
+    to the loaded item.
+
+    \table
+    \row
+    \o sizeloader.qml
+    \o sizeitem.qml
+    \row
+    \o \snippet doc/src/snippets/declarative/loader/sizeloader.qml 0
+    \o \snippet doc/src/snippets/declarative/loader/sizeitem.qml 0
+    \row
+    \o The red rectangle will be sized to the size of the root item.
+    \o The red rectangle will be 50x50, centered in the root item.
+    \endtable
+
 
     \section2 Receiving signals from loaded items
 
@@ -190,7 +226,7 @@ void QDeclarativeLoaderPrivate::initResize()
 */
 
 QDeclarativeLoader::QDeclarativeLoader(QDeclarativeItem *parent)
-  : QDeclarativeItem(*(new QDeclarativeLoaderPrivate), parent)
+  : QDeclarativeImplicitSizeItem(*(new QDeclarativeLoaderPrivate), parent)
 {
     Q_D(QDeclarativeLoader);
     d->flags |= QGraphicsItem::ItemIsFocusScope;
@@ -216,7 +252,8 @@ QDeclarativeLoader::~QDeclarativeLoader()
     cannot load non-visual components.
 
     To unload the currently loaded item, set this property to an empty string,
-    or set \l sourceComponent to \c undefined.
+    or set \l sourceComponent to \c undefined. Setting \c source to a
+    new URL will also cause the item created by the previous URL to be unloaded.
 
     \sa sourceComponent, status, progress
 */
@@ -235,6 +272,7 @@ void QDeclarativeLoader::setSource(const QUrl &url)
     d->clear();
 
     d->source = url;
+
     if (d->source.isEmpty()) {
         emit sourceChanged();
         emit statusChanged();
@@ -245,18 +283,9 @@ void QDeclarativeLoader::setSource(const QUrl &url)
 
     d->component = new QDeclarativeComponent(qmlEngine(this), d->source, this);
     d->ownComponent = true;
-    if (!d->component->isLoading()) {
-        d->_q_sourceLoaded();
-    } else {
-        connect(d->component, SIGNAL(statusChanged(QDeclarativeComponent::Status)),
-                this, SLOT(_q_sourceLoaded()));
-        connect(d->component, SIGNAL(progressChanged(qreal)),
-                this, SIGNAL(progressChanged()));
-        emit statusChanged();
-        emit progressChanged();
-        emit sourceChanged();
-        emit itemChanged();
-    }
+
+    if (isComponentComplete())
+        d->load();
 }
 
 /*!
@@ -297,6 +326,7 @@ void QDeclarativeLoader::setSourceComponent(QDeclarativeComponent *comp)
 
     d->component = comp;
     d->ownComponent = false;
+
     if (!d->component) {
         emit sourceChanged();
         emit statusChanged();
@@ -305,18 +335,8 @@ void QDeclarativeLoader::setSourceComponent(QDeclarativeComponent *comp)
         return;
     }
 
-    if (!d->component->isLoading()) {
-        d->_q_sourceLoaded();
-    } else {
-        connect(d->component, SIGNAL(statusChanged(QDeclarativeComponent::Status)),
-                this, SLOT(_q_sourceLoaded()));
-        connect(d->component, SIGNAL(progressChanged(qreal)),
-                this, SIGNAL(progressChanged()));
-        emit progressChanged();
-        emit sourceChanged();
-        emit statusChanged();
-        emit itemChanged();
-    }
+    if (isComponentComplete())
+        d->load();
 }
 
 void QDeclarativeLoader::resetSourceComponent()
@@ -324,6 +344,27 @@ void QDeclarativeLoader::resetSourceComponent()
     setSourceComponent(0);
 }
 
+void QDeclarativeLoaderPrivate::load()
+{
+    Q_Q(QDeclarativeLoader);
+
+    if (!q->isComponentComplete() || !component)
+        return;
+
+    if (!component->isLoading()) {
+        _q_sourceLoaded();
+    } else {
+        QObject::connect(component, SIGNAL(statusChanged(QDeclarativeComponent::Status)),
+                q, SLOT(_q_sourceLoaded()));
+        QObject::connect(component, SIGNAL(progressChanged(qreal)),
+                q, SIGNAL(progressChanged()));
+        emit q->statusChanged();
+        emit q->progressChanged();
+        emit q->sourceChanged();
+        emit q->itemChanged();
+    }
+}
+
 void QDeclarativeLoaderPrivate::_q_sourceLoaded()
 {
     Q_Q(QDeclarativeLoader);
@@ -337,15 +378,19 @@ void QDeclarativeLoaderPrivate::_q_sourceLoaded()
             return;
         }
 
-        QDeclarativeContext *ctxt = new QDeclarativeContext(qmlContext(q));
+        QDeclarativeContext *creationContext = component->creationContext();
+        if (!creationContext) creationContext = qmlContext(q);
+        QDeclarativeContext *ctxt = new QDeclarativeContext(creationContext);
         ctxt->setContextObject(q);
 
-        QDeclarativeComponent *c = component;
-        QObject *obj = component->create(ctxt);
+        QDeclarativeGuard<QDeclarativeComponent> c = component;
+        QObject *obj = component->beginCreate(ctxt);
         if (component != c) {
             // component->create could trigger a change in source that causes
             // component to be set to something else. In that case we just
             // need to cleanup.
+            if (c)
+                c->completeCreate();
             delete obj;
             delete ctxt;
             return;
@@ -370,6 +415,7 @@ void QDeclarativeLoaderPrivate::_q_sourceLoaded()
             delete ctxt;
             source = QUrl();
         }
+        component->completeCreate();
         emit q->sourceChanged();
         emit q->statusChanged();
         emit q->progressChanged();
@@ -392,23 +438,25 @@ void QDeclarativeLoaderPrivate::_q_sourceLoaded()
     Use this status to provide an update or respond to the status change in some way.
     For example, you could:
 
-    \e {Trigger a state change:}
-    \qml 
-        State { name: 'loaded'; when: loader.status = Loader.Ready }
+    \list
+    \o Trigger a state change:
+    \qml
+        State { name: 'loaded'; when: loader.status == Loader.Ready }
     \endqml
 
-    \e {Implement an \c onStatusChanged signal handler:}
-    \qml 
+    \o Implement an \c onStatusChanged signal handler:
+    \qml
         Loader {
             id: loader
             onStatusChanged: if (loader.status == Loader.Ready) console.log('Loaded')
         }
     \endqml
 
-    \e {Bind to the status value:}
+    \o Bind to the status value:
     \qml
-        Text { text: loader.status != Loader.Ready ? 'Not Loaded' : 'Loaded' }
+        Text { text: loader.status == Loader.Ready ? 'Loaded' : 'Not loaded' }
     \endqml
+    \endlist
 
     Note that if the source is a local file, the status will initially be Ready (or Error). While
     there will be no onStatusChanged signal in that case, the onLoaded will still be invoked.
@@ -431,9 +479,10 @@ QDeclarativeLoader::Status QDeclarativeLoader::status() const
 
 void QDeclarativeLoader::componentComplete()
 {
+    Q_D(QDeclarativeLoader);
+
     QDeclarativeItem::componentComplete();
-    if (status() == Ready)
-        emit loaded();
+    d->load();
 }
 
 
@@ -470,13 +519,21 @@ qreal QDeclarativeLoader::progress() const
 void QDeclarativeLoaderPrivate::_q_updateSize(bool loaderGeometryChanged)
 {
     Q_Q(QDeclarativeLoader);
-    if (!item)
+    if (!item || updatingSize)
         return;
+
+    updatingSize = true;
     if (QDeclarativeItem *qmlItem = qobject_cast<QDeclarativeItem*>(item)) {
-        q->setImplicitWidth(qmlItem->width());
+        if (!itemWidthValid)
+            q->setImplicitWidth(qmlItem->implicitWidth());
+        else
+            q->setImplicitWidth(qmlItem->width());
         if (loaderGeometryChanged && q->widthValid())
             qmlItem->setWidth(q->width());
-        q->setImplicitHeight(qmlItem->height());
+        if (!itemHeightValid)
+            q->setImplicitHeight(qmlItem->implicitHeight());
+        else
+            q->setImplicitHeight(qmlItem->height());
         if (loaderGeometryChanged && q->heightValid())
             qmlItem->setHeight(q->height());
     } else if (item && item->isWidget()) {
@@ -491,6 +548,7 @@ void QDeclarativeLoaderPrivate::_q_updateSize(bool loaderGeometryChanged)
         if (widget->size() != widgetSize)
             widget->resize(widgetSize);
     }
+    updatingSize = false;
 }
 
 /*!
