@@ -1,17 +1,18 @@
 /****************************************************************************
 **
-** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the QtOpenGL module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -21,8 +22,8 @@
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
@@ -33,8 +34,7 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
+**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -68,7 +68,8 @@ enum EngineMode {
     ImageDrawingMode,
     TextDrawingMode,
     BrushDrawingMode,
-    ImageArrayDrawingMode
+    ImageArrayDrawingMode,
+    ImageArrayWithOpacityDrawingMode
 };
 
 QT_BEGIN_NAMESPACE
@@ -126,6 +127,8 @@ public:
     virtual void drawPixmap(const QRectF &r, const QPixmap &pm, const QRectF &sr);
     virtual void drawPixmapFragments(const QPainter::PixmapFragment *fragments, int fragmentCount, const QPixmap &pixmap,
                                      QPainter::PixmapFragmentHints hints);
+    virtual void drawPixmapFragments(const QRectF *targetRects, const QRectF *sourceRects, int fragmentCount, const QPixmap &pixmap,
+                                     QPainter::PixmapFragmentHints hints);
     virtual void drawImage(const QRectF &r, const QImage &pm, const QRectF &sr,
                            Qt::ImageConversionFlags flags = Qt::AutoColor);
     virtual void drawTextItem(const QPointF &p, const QTextItem &textItem);
@@ -158,6 +161,7 @@ public:
     void setRenderTextActive(bool);
 
     bool isNativePaintingActive() const;
+    bool supportsTransformations(qreal, const QTransform &) const { return true; }
 private:
     Q_DISABLE_COPY(QGL2PaintEngineEx)
 };
@@ -181,7 +185,6 @@ public:
             elementIndicesVBOId(0),
             opacityArray(0),
             snapToPixelGrid(false),
-            addOffset(false),
             nativePaintingActive(false),
             inverseScale(1),
             lastMaskTextureUsed(0)
@@ -202,7 +205,9 @@ public:
     void fill(const QVectorPath &path);
     void stroke(const QVectorPath &path, const QPen &pen);
     void drawTexture(const QGLRect& dest, const QGLRect& src, const QSize &textureSize, bool opaque, bool pattern = false);
-    void drawPixmapFragments(const QPainter::PixmapFragment *fragments, int fragmentCount, const QPixmap &pixmap,
+    void drawPixmapFragments(const QPainter::PixmapFragment *fragments, int fragmentCount, const QPixmap &pixmap, const QSize &size,
+                             QPainter::PixmapFragmentHints hints);
+    void drawPixmapFragments(const QRectF *targetRects, const QRectF *sourceRects, int fragmentCount, const QPixmap &pixmap, const QSize &size,
                              QPainter::PixmapFragmentHints hints);
     void drawCachedGlyphs(QFontEngineGlyphCache::Type glyphType, QStaticTextItem *staticTextItem);
 
@@ -255,6 +260,7 @@ public:
     int width, height;
     QGLContext *ctx;
     EngineMode mode;
+    bool imageDrawingMode;
     QFontEngineGlyphCache::Type glyphCacheType;
 
     // Dirty flags
@@ -274,6 +280,8 @@ public:
     QBrush currentBrush; // May not be the state's brush!
     const QBrush noBrush;
 
+    QPixmap currentBrushPixmap;
+
     QGL2PEXVertexArray vertexCoordinateArray;
     QGL2PEXVertexArray textureCoordinateArray;
     QVector<GLushort> elementIndices;
@@ -283,7 +291,6 @@ public:
     GLfloat staticTextureCoordinateArray[8];
 
     bool snapToPixelGrid;
-    bool addOffset; // When enabled, adds a 0.49,0.49 offset to matrix in updateMatrix
     bool nativePaintingActive;
     GLfloat pmvMatrix[3][3];
     GLfloat inverseScale;
