@@ -1,17 +1,18 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the tools applications of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -21,8 +22,8 @@
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
@@ -33,8 +34,7 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
+**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -169,7 +169,7 @@ QVariant DomTool::elementToVariant(const QDomElement& e, const QVariant& defValu
             n3 = n3.nextSibling().toElement();
         }
         var.createRect(x, y, w, h);
-        qVariantSetValue(v, var);
+        v.setValue(var);
     } else if (e.tagName() == QLatin1String("point")) {
         QDomElement n3 = e.firstChild().toElement();
         int x = 0, y = 0;
@@ -181,7 +181,7 @@ QVariant DomTool::elementToVariant(const QDomElement& e, const QVariant& defValu
             n3 = n3.nextSibling().toElement();
         }
         var.createPoint(x,y);
-        qVariantSetValue(v, var);
+        v.setValue(var);
     } else if (e.tagName() == QLatin1String("size")) {
         QDomElement n3 = e.firstChild().toElement();
         int w = 0, h = 0;
@@ -193,10 +193,10 @@ QVariant DomTool::elementToVariant(const QDomElement& e, const QVariant& defValu
             n3 = n3.nextSibling().toElement();
         }
         var.createSize(w, h);
-        qVariantSetValue(v, var);
+        v.setValue(var);
     } else if (e.tagName() == QLatin1String("color")) {
         var.color = readColor(e);
-        qVariantSetValue(v, var);
+        v.setValue(var);
     } else if (e.tagName() == QLatin1String("font")) {
         QDomElement n3 = e.firstChild().toElement();
         Font f;
@@ -217,7 +217,7 @@ QVariant DomTool::elementToVariant(const QDomElement& e, const QVariant& defValu
             n3 = n3.nextSibling().toElement();
         }
         var.font = f;
-        qVariantSetValue(v, var);
+        v.setValue(var);
     } else if (e.tagName() == QLatin1String("string")) {
         v = QVariant(e.firstChild().toText().data());
         QDomElement n = e;
@@ -258,10 +258,10 @@ QVariant DomTool::elementToVariant(const QDomElement& e, const QVariant& defValu
                 var.sizePolicy.verstretch = n3.firstChild().toText().data().toInt();
             n3 = n3.nextSibling().toElement();
         }
-        qVariantSetValue(v, var);
+        v.setValue(var);
     } else if (e.tagName() == QLatin1String("cursor")) {
         var.createCursor(e.firstChild().toText().data().toInt());
-        qVariantSetValue(v, var);
+        v.setValue(var);
     } else if (e.tagName() == QLatin1String("stringlist")) {
         QStringList lst;
         QDomElement n;
