@@ -1,17 +1,18 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the Qt Designer of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -21,8 +22,8 @@
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
@@ -33,8 +34,7 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
+**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -184,15 +184,15 @@ const QByteArray *QtResourceModelPrivate::createResource(const QString &path, QS
             break;
         // return code cannot be fully trusted, might still be empty
         const ResourceDataFileMap resMap = library.resourceDataFileMap();
-        if (resMap.empty())
-            break;
-
         if (!library.output(buffer, errorDevice))
             break;
 
         *errorCount = library.failedResources().size();
         *contents = resMap.keys();
 
+        if (resMap.empty())
+            break;
+
         buffer.close();
         rc = new QByteArray(buffer.data());
     } while (false);
@@ -225,15 +225,18 @@ void QtResourceModelPrivate::registerResourceSet(QtResourceSet *resourceSet)
             qDebug() << "registerResourceSet " << path;
         const PathDataMap::const_iterator itRcc = m_pathToData.constFind(path);
         if (itRcc != m_pathToData.constEnd()) { // otherwise data was not created yet
-            if (!QResource::registerResource(reinterpret_cast<const uchar *>(itRcc.value()->constData()))) {
-                qDebug() << "** WARNING: Failed to register " << path << " (QResource failure).";
-            } else {
-                QStringList contents = m_pathToContents.value(path);
-                QStringListIterator itContents(contents);
-                while (itContents.hasNext()) {
-                    const QString filePath = itContents.next();
-                    if (!m_fileToQrc.contains(filePath)) // the first loaded resource has higher priority in qt resource system
-                        m_fileToQrc.insert(filePath, path);
+            const QByteArray *data = itRcc.value();
+            if (data) {
+                if (!QResource::registerResource(reinterpret_cast<const uchar *>(data->constData()))) {
+                    qWarning() << "** WARNING: Failed to register " << path << " (QResource failure).";
+                } else {
+                    QStringList contents = m_pathToContents.value(path);
+                    QStringListIterator itContents(contents);
+                    while (itContents.hasNext()) {
+                        const QString filePath = itContents.next();
+                        if (!m_fileToQrc.contains(filePath)) // the first loaded resource has higher priority in qt resource system
+                            m_fileToQrc.insert(filePath, path);
+                    }
                 }
             }
         }
@@ -254,8 +257,11 @@ void QtResourceModelPrivate::unregisterResourceSet(QtResourceSet *resourceSet)
             qDebug() << "unregisterResourceSet " << path;
         const PathDataMap::const_iterator itRcc = m_pathToData.constFind(path);
         if (itRcc != m_pathToData.constEnd()) { // otherwise data was not created yet
-            if (!QResource::unregisterResource(reinterpret_cast<const uchar *>(itRcc.value()->constData())))
-                qDebug() << "** WARNING: Failed to unregister " << path << " (QResource failure).";
+            const QByteArray *data = itRcc.value();
+            if (data) {
+                if (!QResource::unregisterResource(reinterpret_cast<const uchar *>(itRcc.value()->constData())))
+                    qWarning() << "** WARNING: Failed to unregister " << path << " (QResource failure).";
+            }
         }
     }
     m_fileToQrc.clear();
@@ -292,15 +298,13 @@ void QtResourceModelPrivate::activate(QtResourceSet *resourceSet, const QStringL
             QStringList contents;
             int qrcErrorCount;
             generatedCount++;
-            if (const QByteArray *data = createResource(path, &contents, &qrcErrorCount, errorStream)) {
-                newPathToData.insert(path, data);
-                if (qrcErrorCount) // Count single failed files as sort of 1/2 error
-                    errorCount++;
-                addWatcher(path);
-            } else {
-                newPathToData.remove(path);
+            const QByteArray *data = createResource(path, &contents, &qrcErrorCount, errorStream);
+
+            newPathToData.insert(path, data);
+            if (qrcErrorCount) // Count single failed files as sort of 1/2 error
                 errorCount++;
-            }
+            addWatcher(path);
+
             m_pathToModified.insert(path, false);
             m_pathToContents.insert(path, contents);
             newResourceSetChanged = true;
@@ -326,7 +330,7 @@ void QtResourceModelPrivate::activate(QtResourceSet *resourceSet, const QStringL
     QListIterator<const QByteArray *> itOld(oldData);
     if (itOld.hasNext()) {
         const QByteArray *array = itOld.next();
-        if (!newData.contains(array))
+        if (array && !newData.contains(array))
             toDelete.append(array);
     }
 
@@ -428,10 +432,10 @@ void QtResourceModelPrivate::removeOldPaths(QtResourceSet *resourceSet, const QS
 
 void QtResourceModelPrivate::setWatcherEnabled(const QString &path, bool enable)
 {
-    m_fileWatcher->removePath(path);
-
-    if (!enable)
+    if (!enable) {
+        m_fileWatcher->removePath(path);
         return;
+    }
 
     QFileInfo fi(path);
     if (fi.exists())
@@ -476,8 +480,8 @@ QtResourceModel::QtResourceModel(QObject *parent) :
     d_ptr->q_ptr = this;
 
     d_ptr->m_fileWatcher = new QFileSystemWatcher(this);
-    connect(d_ptr->m_fileWatcher, SIGNAL(fileChanged(const QString &)),
-            this, SLOT(slotFileChanged(const QString &)));
+    connect(d_ptr->m_fileWatcher, SIGNAL(fileChanged(QString)),
+            this, SLOT(slotFileChanged(QString)));
 }
 
 QtResourceModel::~QtResourceModel()
