@@ -1,18 +1,19 @@
 /****************************************************************************
 **
 ** Copyright (C) 2001-2004 Roberto Raggi
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the qt3to4 porting application of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -22,8 +23,8 @@
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
@@ -34,8 +35,7 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
+**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -344,27 +344,27 @@ void Semantic::parseDeclaration(AST *funSpec, AST *storageSpec, TypeSpecifierAST
 void Semantic::parseFunctionDeclaration(AST *funSpec, AST *storageSpec,
                                         TypeSpecifierAST * typeSpec, InitDeclaratorAST * initDeclarator)
 {
-    bool isFriend = false;
     bool isVirtual = false;
     bool isStatic = false;
-    bool isInline = false;
     bool isPure = initDeclarator->initializer() != 0;
 
     if (funSpec){
         List<AST*> l = *funSpec->children();
         foreach (AST *current, l) {
-            QByteArray text = textOf(current);
-            if (text == "virtual") isVirtual = true;
-            else if (text == "inline") isInline = true;
+            if (textOf(current) == "virtual") {
+                isVirtual = true;
+                break;
+            }
         }
     }
 
     if (storageSpec){
         List<AST*> l = *storageSpec->children();
         foreach (AST *current, l) {
-            QByteArray text = textOf(current);
-            if (text == "friend") isFriend = true;
-            else if (text == "static") isStatic = true;
+            if (textOf(current) == "static") {
+                isStatic = true;
+                break;
+            }
         }
     }
     DeclaratorAST *declarator = initDeclarator->declarator();
@@ -768,7 +768,7 @@ void Semantic::parseTypedef(TypedefAST *ast)
 void Semantic::parseTypeSpecifier(TypeSpecifierAST *ast)
 {
     // If this is a classSpecifier or a EnumSpecifier we skip the name lookup,
-    // becuase looking up the name "E" in a class definition like
+    // because looking up the name "E" in a class definition like
     // "class E { ..." makes no sense. (There might be a variable named E
     // already declared, but that variable is now shadowed by the class type.)
     if(   ast->nodeType() != NodeType_EnumSpecifier
@@ -806,10 +806,10 @@ void Semantic::parseNameUse(NameAST* name)
 }
 
 /*
-    looks up name used in basescope. If name->isGlobal() is true or if classOrNamespaceList()
-    returns a non-emty list, the C++ qualified name lookup rules are used. Otherwise the
+    Looks up name used in basescope. If name->isGlobal() is true or if classOrNamespaceList()
+    returns a non-empty list, the C++ qualified name lookup rules are used. Otherwise the
     unquialified name lookup rules are used.  Returns the a list of members that was found,
-    In most cases this list will contain zero or one element, exept in the case of overloaded functions.
+    In most cases this list will contain zero or one element, except in the case of overloaded functions.
     TODO: Argument-dependent name lookup
 */
 QList<CodeModel::Member *> Semantic::nameLookup(CodeModel::Scope *baseScope, const NameAST* name)
@@ -975,7 +975,7 @@ FunctionMember *Semantic::functionLookup(CodeModel::Scope *baseScope,
 */
 FunctionMember *Semantic::selectFunction(QList<CodeModel::Member*> candidatateList, const DeclaratorAST *functionDeclarator)
 {
-    // get arguments for funciton we are looking for
+    // get arguments for function we are looking for
     FunctionMember testFunction;
     parseFunctionArguments(functionDeclarator, &testFunction);
     const ArgumentCollection testArgumentCollection = testFunction.arguments();
