@@ -1,17 +1,18 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the tools applications of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -21,8 +22,8 @@
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
@@ -33,13 +34,11 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
+**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
-#include <QtXml>
 #include <QHash>
 #include <QMap>
 
@@ -49,6 +48,7 @@
 #include "config.h"
 #include "node.h"
 #include "tree.h"
+#include <qdebug.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -91,6 +91,7 @@ HelpProjectWriter::HelpProjectWriter(const Config &config, const QString &defaul
             subproject.title = config.getString(subprefix + "title");
             subproject.indexTitle = config.getString(subprefix + "indexTitle");
             subproject.sortPages = config.getBool(subprefix + "sortPages");
+            subproject.type = config.getString(subprefix + "type");
             readSelectors(subproject, config.getStringList(subprefix + "selectors"));
             project.subprojects[name] = subproject;
         }
@@ -117,6 +118,11 @@ void HelpProjectWriter::readSelectors(SubProject &subproject, const QStringList
     typeHash["property"] = Node::Property;
     typeHash["variable"] = Node::Variable;
     typeHash["target"] = Node::Target;
+#ifdef QDOC_QML
+    typeHash["qmlproperty"] = Node::QmlProperty;
+    typeHash["qmlsignal"] = Node::QmlSignal;
+    typeHash["qmlmethod"] = Node::QmlMethod;
+#endif
 
     QHash<QString, Node::SubType> subTypeHash;
     subTypeHash["example"] = Node::Example;
@@ -128,6 +134,8 @@ void HelpProjectWriter::readSelectors(SubProject &subproject, const QStringList
     subTypeHash["externalpage"] = Node::ExternalPage;
 #ifdef QDOC_QML
     subTypeHash["qmlclass"] = Node::QmlClass;
+    subTypeHash["qmlpropertygroup"] = Node::QmlPropertyGroup;
+    subTypeHash["qmlbasictype"] = Node::QmlBasicType;
 #endif
 
     QSet<Node::SubType> allSubTypes = QSet<Node::SubType>::fromList(subTypeHash.values());
@@ -177,7 +185,13 @@ QStringList HelpProjectWriter::keywordDetails(const Node *node) const
 {
     QStringList details;
 
-    if (node->parent() && !node->parent()->name().isEmpty()) {
+    if (node->type() == Node::QmlProperty) {
+        // "name"
+        details << node->name();
+        // "id"
+        details << node->parent()->parent()->name()+"::"+node->name();
+    }
+    else if (node->parent() && !node->parent()->name().isEmpty()) {
         // "name"
         if (node->type() == Node::Enum || node->type() == Node::Typedef)
             details << node->parent()->name()+"::"+node->name();
@@ -185,29 +199,29 @@ QStringList HelpProjectWriter::keywordDetails(const Node *node) const
             details << node->name();
         // "id"
         details << node->parent()->name()+"::"+node->name();
-    } else if (node->type() == Node::Fake) {
+    }
+    else if (node->type() == Node::Fake) {
         const FakeNode *fake = static_cast<const FakeNode *>(node);
-#ifdef QDOC_QML
         if (fake->subType() == Node::QmlClass) {
             details << (QmlClassNode::qmlOnly ? fake->name() : fake->fullTitle());
             details << "QML." + fake->name();
-        } else
-#endif
-        {
+        }
+        else {
             details << fake->fullTitle();
             details << fake->fullTitle();
         }
-    } else {
+    }
+    else {
         details << node->name();
         details << node->name();
     }
-    details << tree->fullDocumentLocation(node);
-
+    details << HtmlGenerator::fullDocumentLocation(node);
     return details;
 }
 
 bool HelpProjectWriter::generateSection(HelpProject &project,
-                        QXmlStreamWriter & /* writer */, const Node *node)
+                                        QXmlStreamWriter & /* writer */,
+                                        const Node *node)
 {
     if (!node->url().isEmpty())
         return false;
@@ -226,17 +240,19 @@ bool HelpProjectWriter::generateSection(HelpProject &project,
     if (node->type() == Node::Fake) {
         const FakeNode *fake = static_cast<const FakeNode *>(node);
         objName = fake->fullTitle();
-    } else
+    }
+    else
         objName = tree->fullDocumentName(node);
-
+    
     // Only add nodes to the set for each subproject if they match a selector.
     // Those that match will be listed in the table of contents.
 
     foreach (const QString &name, project.subprojects.keys()) {
         SubProject subproject = project.subprojects[name];
         // No selectors: accept all nodes.
-        if (subproject.selectors.isEmpty())
+        if (subproject.selectors.isEmpty()) {
             project.subprojects[name].nodes[objName] = node;
+        }
         else if (subproject.selectors.contains(node->type())) {
             // Accept only the node types in the selectors hash.
             if (node->type() != Node::Fake)
@@ -247,9 +263,10 @@ bool HelpProjectWriter::generateSection(HelpProject &project,
                 const FakeNode *fakeNode = static_cast<const FakeNode *>(node);
                 if (subproject.selectors[node->type()].contains(fakeNode->subType()) &&
                     fakeNode->subType() != Node::ExternalPage &&
-                    !fakeNode->fullTitle().isEmpty())
+                    !fakeNode->fullTitle().isEmpty()) {
 
                     project.subprojects[name].nodes[objName] = node;
+                }
             }
         }
     }
@@ -258,12 +275,12 @@ bool HelpProjectWriter::generateSection(HelpProject &project,
 
         case Node::Class:
             project.keywords.append(keywordDetails(node));
-            project.files.insert(tree->fullDocumentLocation(node));
+            project.files.insert(HtmlGenerator::fullDocumentLocation(node));
             break;
 
         case Node::Namespace:
             project.keywords.append(keywordDetails(node));
-            project.files.insert(tree->fullDocumentLocation(node));
+            project.files.insert(HtmlGenerator::fullDocumentLocation(node));
             break;
 
         case Node::Enum:
@@ -283,13 +300,16 @@ bool HelpProjectWriter::generateSection(HelpProject &project,
                         details << item.name(); // "name"
                         details << item.name(); // "id"
                     }
-                    details << tree->fullDocumentLocation(node);
+                    details << HtmlGenerator::fullDocumentLocation(node);
                     project.keywords.append(details);
                 }
             }
             break;
 
         case Node::Property:
+        case Node::QmlProperty:
+        case Node::QmlSignal:
+        case Node::QmlMethod:
             project.keywords.append(keywordDetails(node));
             break;
 
@@ -311,7 +331,7 @@ bool HelpProjectWriter::generateSection(HelpProject &project,
 
                 if (node->relates()) {
                     project.memberStatus[node->relates()].insert(node->status());
-                    project.files.insert(tree->fullDocumentLocation(node->relates()));
+                    project.files.insert(HtmlGenerator::fullDocumentLocation(node->relates()));
                 } else if (node->parent())
                     project.memberStatus[node->parent()].insert(node->status());
             }
@@ -325,12 +345,20 @@ bool HelpProjectWriter::generateSection(HelpProject &project,
                 // Use the location of any associated enum node in preference
                 // to that of the typedef.
                 if (enumNode)
-                    typedefDetails[2] = tree->fullDocumentLocation(enumNode);
+                    typedefDetails[2] = HtmlGenerator::fullDocumentLocation(enumNode);
 
                 project.keywords.append(typedefDetails);
             }
             break;
 
+        case Node::Variable:
+            {
+                QString location = HtmlGenerator::fullDocumentLocation(node);
+                project.files.insert(location.left(location.lastIndexOf(QLatin1Char('#'))));
+                project.keywords.append(keywordDetails(node));
+            }
+            break;
+
         // Fake nodes (such as manual pages) contain subtypes, titles and other
         // attributes.
         case Node::Fake: {
@@ -345,11 +373,11 @@ bool HelpProjectWriter::generateSection(HelpProject &project,
                                 QStringList details;
                                 details << keyword->string()
                                         << keyword->string()
-                                        << tree->fullDocumentLocation(node) + "#" + Doc::canonicalTitle(keyword->string());
+                                        << HtmlGenerator::fullDocumentLocation(node) + "#" + Doc::canonicalTitle(keyword->string());
                                 project.keywords.append(details);
                             } else
                                 fakeNode->doc().location().warning(
-                                    tr("Bad keyword in %1").arg(tree->fullDocumentLocation(node))
+                                    tr("Bad keyword in %1").arg(HtmlGenerator::fullDocumentLocation(node))
                                     );
                         }
                     }
@@ -363,16 +391,16 @@ bool HelpProjectWriter::generateSection(HelpProject &project,
                             QStringList details;
                             details << title
                                     << title
-                                    << tree->fullDocumentLocation(node) + "#" + Doc::canonicalTitle(title);
+                                    << HtmlGenerator::fullDocumentLocation(node) + "#" + Doc::canonicalTitle(title);
                             project.keywords.append(details);
                         } else
                             fakeNode->doc().location().warning(
-                                tr("Bad contents item in %1").arg(tree->fullDocumentLocation(node))
+                                tr("Bad contents item in %1").arg(HtmlGenerator::fullDocumentLocation(node))
                                 );
                     }
                 }
 */
-                project.files.insert(tree->fullDocumentLocation(node));
+                project.files.insert(HtmlGenerator::fullDocumentLocation(node));
             }
             break;
             }
@@ -400,7 +428,7 @@ void HelpProjectWriter::generateSections(HelpProject &project,
 {
     if (!generateSection(project, writer, node))
         return;
-
+    
     if (node->isInnerNode()) {
         const InnerNode *inner = static_cast<const InnerNode *>(node);
 
@@ -409,8 +437,23 @@ void HelpProjectWriter::generateSections(HelpProject &project,
         foreach (const Node *node, inner->childNodes()) {
             if (node->access() == Node::Private)
                 continue;
-            if (node->type() == Node::Fake)
-                childMap[static_cast<const FakeNode *>(node)->fullTitle()] = node;
+            if (node->type() == Node::Fake) {
+                /*
+                  Don't visit QML property group nodes,
+                  but visit their children, which are all
+                  QML property nodes.
+                 */
+                if (node->subType() == Node::QmlPropertyGroup) {
+                    const InnerNode* inner = static_cast<const InnerNode*>(node);
+                    foreach (const Node* n, inner->childNodes()) {
+                        if (n->access() == Node::Private)
+                            continue;
+                        childMap[tree->fullDocumentName(n)] = n;
+                    }
+                }
+                else
+                    childMap[static_cast<const FakeNode *>(node)->fullTitle()] = node;
+            }
             else {
                 if (node->type() == Node::Function) {
                     const FunctionNode *funcNode = static_cast<const FunctionNode *>(node);
@@ -436,7 +479,7 @@ void HelpProjectWriter::generate(const Tree *tre)
 void HelpProjectWriter::writeNode(HelpProject &project, QXmlStreamWriter &writer,
                                   const Node *node)
 {
-    QString href = tree->fullDocumentLocation(node);
+    QString href = HtmlGenerator::fullDocumentLocation(node);
     QString objName = node->name();
 
     switch (node->type()) {
@@ -494,13 +537,11 @@ void HelpProjectWriter::writeNode(HelpProject &project, QXmlStreamWriter &writer
             writer.writeStartElement("section");
             writer.writeAttribute("ref", href);
             writer.writeAttribute("title", fakeNode->fullTitle());
-            //            qDebug() << "Title:" << fakeNode->fullTitle();
             
-            if (fakeNode->subType() == Node::HeaderFile) {
-
+            if ((fakeNode->subType() == Node::HeaderFile) || (fakeNode->subType() == Node::QmlClass)) {
                 // Write subsections for all members, obsolete members and Qt 3
                 // members.
-                if (!project.memberStatus[node].isEmpty()) {
+                if (!project.memberStatus[node].isEmpty() || (fakeNode->subType() == Node::QmlClass)) {
                     QString membersPath = href.left(href.size()-5) + "-members.html";
                     writer.writeStartElement("section");
                     writer.writeAttribute("ref", membersPath);
@@ -581,56 +622,115 @@ void HelpProjectWriter::generateProject(HelpProject &project)
 
     writer.writeStartElement("toc");
     writer.writeStartElement("section");
-    QString indexPath = tree->fullDocumentLocation(tree->findFakeNodeByTitle(project.indexTitle));
+    QString indexPath = HtmlGenerator::fullDocumentLocation(tree->findFakeNodeByTitle(project.indexTitle));
     if (indexPath.isEmpty())
         indexPath = "index.html";
     writer.writeAttribute("ref", HtmlGenerator::cleanRef(indexPath));
     writer.writeAttribute("title", project.indexTitle);
-    project.files.insert(tree->fullDocumentLocation(rootNode));
+    project.files.insert(HtmlGenerator::fullDocumentLocation(rootNode));
 
     generateSections(project, writer, rootNode);
 
     foreach (const QString &name, project.subprojects.keys()) {
         SubProject subproject = project.subprojects[name];
 
-        if (!name.isEmpty()) {
-            writer.writeStartElement("section");
-            QString indexPath = tree->fullDocumentLocation(tree->findFakeNodeByTitle(subproject.indexTitle));
-            writer.writeAttribute("ref", HtmlGenerator::cleanRef(indexPath));
-            writer.writeAttribute("title", subproject.title);
-            project.files.insert(indexPath);
-        }
-        if (subproject.sortPages) {
-            QStringList titles = subproject.nodes.keys();
-            titles.sort();
-            foreach (const QString &title, titles)
-                writeNode(project, writer, subproject.nodes[title]);
-        } else {
-            // Find a contents node and navigate from there, using the NextLink values.
-            foreach (const Node *node, subproject.nodes) {
-                QString nextTitle = node->links().value(Node::NextLink).first;
-                if (!nextTitle.isEmpty() &&
-                    node->links().value(Node::ContentsLink).first.isEmpty()) {
+        if (subproject.type == QLatin1String("manual")) {
 
-                    FakeNode *nextPage = const_cast<FakeNode *>(tree->findFakeNodeByTitle(nextTitle));
+            const FakeNode *indexPage = tree->findFakeNodeByTitle(subproject.indexTitle);
+            if (indexPage) {
+                Text indexBody = indexPage->doc().body();
+                const Atom *atom = indexBody.firstAtom();
+                QStack<int> sectionStack;
+                bool inItem = false;
 
-                    // Write the contents node.
-                    writeNode(project, writer, node);
+                while (atom) {
+                    switch (atom->type()) {
+                    case Atom::ListLeft:
+                        sectionStack.push(0);
+                        break;
+                    case Atom::ListRight:
+                        if (sectionStack.pop() > 0)
+                            writer.writeEndElement(); // section
+                        break;
+                    case Atom::ListItemLeft:
+                        inItem = true;
+                        break;
+                    case Atom::ListItemRight:
+                        inItem = false;
+                        break;
+                    case Atom::Link:
+                        if (inItem) {
+                            if (sectionStack.top() > 0)
+                                writer.writeEndElement(); // section
 
-                    while (nextPage) {
-                        writeNode(project, writer, nextPage);
-                        nextTitle = nextPage->links().value(Node::NextLink).first;
-                        if(nextTitle.isEmpty())
-                            break;
-                        nextPage = const_cast<FakeNode *>(tree->findFakeNodeByTitle(nextTitle));
+                            const FakeNode *page = tree->findFakeNodeByTitle(atom->string());
+                            writer.writeStartElement("section");
+                            QString indexPath = HtmlGenerator::fullDocumentLocation(page);
+                            writer.writeAttribute("ref", HtmlGenerator::cleanRef(indexPath));
+                            writer.writeAttribute("title", atom->string());
+                            project.files.insert(indexPath);
+
+                            sectionStack.top() += 1;
+                        }
+                        break;
+                    default:
+                        ;
+                    }
+
+                    if (atom == indexBody.lastAtom())
+                        break;
+                    atom = atom->next();
+                }
+            } else
+                rootNode->doc().location().warning(
+                    tr("Failed to find index: %1").arg(subproject.indexTitle)
+                    );
+
+        } else {
+
+            if (!name.isEmpty()) {
+                writer.writeStartElement("section");
+                QString indexPath = HtmlGenerator::fullDocumentLocation(tree->findFakeNodeByTitle(subproject.indexTitle));
+                writer.writeAttribute("ref", HtmlGenerator::cleanRef(indexPath));
+                writer.writeAttribute("title", subproject.title);
+                project.files.insert(indexPath);
+            }
+            if (subproject.sortPages) {
+                QStringList titles = subproject.nodes.keys();
+                titles.sort();
+                foreach (const QString &title, titles) {
+                    writeNode(project, writer, subproject.nodes[title]);
+                }
+            } else {
+                // Find a contents node and navigate from there, using the NextLink values.
+                QSet<QString> visited;
+
+                foreach (const Node *node, subproject.nodes) {
+                    QString nextTitle = node->links().value(Node::NextLink).first;
+                    if (!nextTitle.isEmpty() &&
+                        node->links().value(Node::ContentsLink).first.isEmpty()) {
+
+                        FakeNode *nextPage = const_cast<FakeNode *>(tree->findFakeNodeByTitle(nextTitle));
+
+                        // Write the contents node.
+                        writeNode(project, writer, node);
+
+                        while (nextPage) {
+                            writeNode(project, writer, nextPage);
+                            nextTitle = nextPage->links().value(Node::NextLink).first;
+                            if (nextTitle.isEmpty() || visited.contains(nextTitle))
+                                break;
+                            nextPage = const_cast<FakeNode *>(tree->findFakeNodeByTitle(nextTitle));
+                            visited.insert(nextTitle);
+                        }
+                        break;
                     }
-                    break;
                 }
             }
-        }
 
-        if (!name.isEmpty())
-            writer.writeEndElement(); // section
+            if (!name.isEmpty())
+                writer.writeEndElement(); // section
+        }
     }
 
     writer.writeEndElement(); // section
