@@ -1,17 +1,18 @@
 /****************************************************************************
 **
-** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the tools applications of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -21,8 +22,8 @@
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
@@ -33,8 +34,7 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
+**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -46,10 +46,13 @@
 #include "node.h"
 #include "tree.h"
 #include "codemarker.h"
+#include <QUuid>
 #include <qdebug.h>
 
 QT_BEGIN_NAMESPACE
 
+ExampleNodeMap ExampleNode::exampleNodeMap;
+
 /*!
   \class Node
   \brief The Node class is a node in the Tree.
@@ -103,7 +106,6 @@ Node::Node(Type type, InnerNode *parent, const QString& name)
 {
     if (par)
         par->addChild(this);
-    //uuid = QUuid::createUuid();
 }
 
 /*!
@@ -156,6 +158,16 @@ void Node::setLink(LinkType linkType, const QString &link, const QString &desc)
     linkMap[linkType] = linkPair;
 }
 
+/*!
+    Sets the information about the project and version a node was introduced
+    in. The string is simplified, removing excess whitespace before being
+    stored.
+*/
+void Node::setSince(const QString &since)
+{
+    sinc = since.simplified();
+}
+
 /*!
   Returns a string representing the access specifier.
  */
@@ -270,14 +282,16 @@ QString Node::fileBase() const
 }
 
 /*!
-  Returns this node's Universally Unique IDentifier.
-  If its UUID has not yet been created, it is created
-  first.
+  Returns this node's Universally Unique IDentifier as a
+  QString. Creates the UUID first, if it has not been created.
  */
-QUuid Node::guid() const
+QString Node::guid() const
 {
-    if (uuid.isNull())
-        uuid = QUuid::createUuid();
+    if (uuid.isEmpty()) {
+        QUuid quuid = QUuid::createUuid();
+        QString t = quuid.toString();
+        uuid = "id-" + t.mid(1,t.length()-2);
+    }
     return uuid;
 }
 
@@ -304,6 +318,38 @@ QString Node::ditaXmlHref()
     return href + "#" + guid();
 }
 
+/*!
+  If this node is a QML class node, return a pointer to it.
+  If it is a child of a QML class node, return a pointer to
+  the QML class node. Otherwise, return 0;
+ */
+const QmlClassNode* Node::qmlClassNode() const
+{
+    if (isQmlNode()) {
+        const Node* n = this;
+        while (n && n->subType() != Node::QmlClass)
+            n = n->parent();
+        if (n && n->subType() == Node::QmlClass)
+            return static_cast<const QmlClassNode*>(n);
+    }
+    return 0;
+}
+
+/*!
+  If this node is a QML node, find its QML class node,
+  and return a pointer to the C++ class node from the
+  QML class node. That pointer will be null if the QML
+  class node is a component. It will be non-null if
+  the QML class node is a QML element.
+ */
+const ClassNode* Node::declarativeCppNode() const
+{
+    const QmlClassNode* qcn = qmlClassNode();
+    if (qcn)
+        return qcn->classNode();
+    return 0;
+}
+
 /*!
   \class InnerNode
  */
@@ -543,7 +589,8 @@ void InnerNode::removeFromRelated()
  */
 void InnerNode::deleteChildren()
 {
-    qDeleteAll(children);
+    NodeList childrenCopy = children; // `children` will be changed in ~Node()
+    qDeleteAll(childrenCopy);
 }
 
 /*!
@@ -945,10 +992,35 @@ void ClassNode::fixBaseClasses()
   Search the child list to find the property node with the
   specified \a name.
  */
-const PropertyNode* ClassNode::findPropertyNode(const QString& name) const
+const PropertyNode *ClassNode::findPropertyNode(const QString &name) const
 {
-    const Node* n = findNode(name,Node::Property);
-    return (n ? static_cast<const PropertyNode*>(n) : 0);
+    const Node *n = findNode(name, Node::Property);
+
+    if (n)
+        return static_cast<const PropertyNode*>(n);
+
+    const PropertyNode *pn = 0;
+
+    const QList<RelatedClass> &bases = baseClasses();
+    if (!bases.isEmpty()) {
+        for (int i = 0; i < bases.size(); ++i) {
+            const ClassNode *cn = bases[i].node;
+            pn = cn->findPropertyNode(name);
+            if (pn)
+                break;
+        }
+    }
+    const QList<RelatedClass>& ignoredBases = ignoredBaseClasses();
+    if (!ignoredBases.isEmpty()) {
+        for (int i = 0; i < ignoredBases.size(); ++i) {
+            const ClassNode *cn = ignoredBases[i].node;
+            pn = cn->findPropertyNode(name);
+            if (pn)
+                break;
+        }
+    }
+
+    return pn;
 }
 
 /*!
@@ -1034,6 +1106,16 @@ QString FakeNode::subTitle() const
     return QString();
 }
 
+/*!
+  The constructor calls the FakeNode constructor with
+  \a parent, \a name, and Node::Example.
+ */
+ExampleNode::ExampleNode(InnerNode* parent, const QString& name)
+    : FakeNode(parent, name, Node::Example)
+{
+    // nothing
+}
+
 /*!
   \class EnumNode
  */
@@ -1152,8 +1234,8 @@ QString Parameter::reconstruct(bool value) const
     if (!p.endsWith(QChar('*')) && !p.endsWith(QChar('&')) && !p.endsWith(QChar(' ')))
         p += " ";
     p += nam;
-    if (value)
-        p += def;
+    if (value && !def.isEmpty())
+        p += " = " + def;
     return p;
 }
 
@@ -1361,6 +1443,21 @@ QString FunctionNode::signature(bool values) const
     return s;
 }
 
+/*!
+  Returns true if the node's status is Internal, or if its
+  parent is a class with internal status.
+ */
+bool FunctionNode::isInternal() const
+{
+    if (status() == Internal)
+        return true;
+    if (parent() && parent()->status() == Internal)
+        return true;
+    if (relates() && relates()->status() == Internal)
+        return true;
+    return false;
+}
+
 /*!
   Print some debugging stuff.
  */
@@ -1389,6 +1486,7 @@ PropertyNode::PropertyNode(InnerNode *parent, const QString& name)
       usr(Trool_Default),
       cst(false),
       fnl(false),
+      rev(-1),
       overrides(0)
 {
     // nothing.
@@ -1545,11 +1643,6 @@ void QmlClassNode::clear()
  */
 QString QmlClassNode::fileBase() const
 {
-#if 0
-    if (Node::fileBase() == "item")
-        qDebug() << "FILEBASE: qmlitem" << name();
-    return "qml_" + Node::fileBase();
-#endif
     return Node::fileBase();
 }
 
@@ -1645,46 +1738,31 @@ bool QmlPropertyNode::fromTrool(Trool troolean, bool defaultValue)
     }
 }
 
-static QString valueType(const QString& n)
-{
-    if (n == "QPoint")
-        return "QDeclarativePointValueType";
-    if (n == "QPointF")
-        return "QDeclarativePointFValueType";
-    if (n == "QSize")
-        return "QDeclarativeSizeValueType";
-    if (n == "QSizeF")
-        return "QDeclarativeSizeFValueType";
-    if (n == "QRect")
-        return "QDeclarativeRectValueType";
-    if (n == "QRectF")
-        return "QDeclarativeRectFValueType";
-    if (n == "QVector2D")
-        return "QDeclarativeVector2DValueType";
-    if (n == "QVector3D")
-        return "QDeclarativeVector3DValueType";
-    if (n == "QVector4D")
-        return "QDeclarativeVector4DValueType";
-    if (n == "QQuaternion")
-        return "QDeclarativeQuaternionValueType";
-    if (n == "QMatrix4x4")
-        return "QDeclarativeMatrix4x4ValueType";
-    if (n == "QEasingCurve")
-        return "QDeclarativeEasingValueType";
-    if (n == "QFont")
-        return "QDeclarativeFontValueType";
-    return QString();
-}
-
 /*!
   Returns true if a QML property or attached property is
   read-only. The algorithm for figuring this out is long
   amd tedious and almost certainly will break. It currently
   doesn't work for qmlproperty bool PropertyChanges::explicit,
-  because the tokenized gets confused on "explicit" .
+  because the tokenizer gets confused on "explicit".
  */
 bool QmlPropertyNode::isWritable(const Tree* tree) const
 {
+    if (wri != Trool_Default)
+        return fromTrool(wri, false);
+
+    const PropertyNode *pn = correspondingProperty(tree);
+    if (pn)
+        return pn->isWritable();
+    else {
+        location().warning(tr("Can't determine read-only status of QML property %1; writable assumed.").arg(name()));
+        return true;
+    }
+}
+
+const PropertyNode *QmlPropertyNode::correspondingProperty(const Tree *tree) const
+{
+    const PropertyNode *pn;
+
     Node* n = parent();
     while (n && n->subType() != Node::QmlClass)
         n = n->parent();
@@ -1693,93 +1771,35 @@ bool QmlPropertyNode::isWritable(const Tree* tree) const
         const ClassNode* cn = qcn->classNode();
         if (cn) {
             QStringList dotSplit = name().split(QChar('.'));
-            const PropertyNode* pn = cn->findPropertyNode(dotSplit[0]);
+            pn = cn->findPropertyNode(dotSplit[0]);
             if (pn) {
                 if (dotSplit.size() > 1) {
+                    // Find the C++ property corresponding to the QML property in
+                    // the property group, <group>.<property>.
+
                     QStringList path(extractClassName(pn->qualifiedDataType()));
                     const Node* nn = tree->findNode(path,Class);
                     if (nn) {
                         const ClassNode* cn = static_cast<const ClassNode*>(nn);
-                        pn = cn->findPropertyNode(dotSplit[1]);
-                        if (pn) {
-                            return pn->isWritable();
-                        }
-                        else {
-                            const QList<RelatedClass>& bases = cn->baseClasses();
-                            if (!bases.isEmpty()) {
-                                for (int i=0; i<bases.size(); ++i) {
-                                    const ClassNode* cn = bases[i].node;
-                                    pn = cn->findPropertyNode(dotSplit[1]);
-                                    if (pn) {
-                                        return pn->isWritable();
-                                    }
-                                }
-                            }
-                            const QList<RelatedClass>& ignoredBases = cn->ignoredBaseClasses();
-                            if (!ignoredBases.isEmpty()) {
-                                for (int i=0; i<ignoredBases.size(); ++i) {
-                                    const ClassNode* cn = ignoredBases[i].node;
-                                    pn = cn->findPropertyNode(dotSplit[1]);
-                                    if (pn) {
-                                        return pn->isWritable();
-                                    }
-                                }
-                            }
-                            QString vt = valueType(cn->name());
-                            if (!vt.isEmpty()) {
-                                QStringList path(vt);
-                                const Node* vtn = tree->findNode(path,Class);
-                                if (vtn) {
-                                    const ClassNode* cn = static_cast<const ClassNode*>(vtn);
-                                    pn = cn->findPropertyNode(dotSplit[1]);
-                                    if (pn) {
-                                        return pn->isWritable();
-                                    }
-                                }
-                            }
-                        }
+                        const PropertyNode *pn2 = cn->findPropertyNode(dotSplit[1]);
+                        if (pn2)
+                            return pn2; // Return the property for the QML property.
+                        else
+                            return pn;  // Return the property for the QML group.
                     }
                 }
-                else {
-                    return pn->isWritable();
-                }
+                else
+                    return pn;
             }
             else {
-                const QList<RelatedClass>& bases = cn->baseClasses();
-                if (!bases.isEmpty()) {
-                    for (int i=0; i<bases.size(); ++i) {
-                        const ClassNode* cn = bases[i].node;
-                        pn = cn->findPropertyNode(dotSplit[0]);
-                        if (pn) {
-                            return pn->isWritable();
-                        }
-                    }
-                }
-                const QList<RelatedClass>& ignoredBases = cn->ignoredBaseClasses();
-                if (!ignoredBases.isEmpty()) {
-                    for (int i=0; i<ignoredBases.size(); ++i) {
-                        const ClassNode* cn = ignoredBases[i].node;
-                        pn = cn->findPropertyNode(dotSplit[0]);
-                        if (pn) {
-                            return pn->isWritable();
-                        }
-                    }
-                }
-                if (isAttached()) {
-                    QString classNameAttached = cn->name() + "Attached";
-                    QStringList path(classNameAttached);
-                    const Node* nn = tree->findNode(path,Class);
-                    const ClassNode* acn = static_cast<const ClassNode*>(nn);
-                    pn = acn->findPropertyNode(dotSplit[0]);
-                    if (pn) {
-                        return pn->isWritable();
-                    }
-                }
+                pn = cn->findPropertyNode(dotSplit[0]);
+                if (pn)
+                    return pn;
             }
         }
     }
-    location().warning(tr("Can't determine read-only status of QML property %1; writable assumed.").arg(name()));
-    return true;
+
+    return 0;
 }
 
 #endif
